/****************************************************************
http://social.msdn.microsoft.com/Forums/en-US/transactsql/thread/ab90a200-4a1d-4aa3-a085-9d7cd318a76c

Solution Preparation:
1. Create a temp table #BridgeP
2. Create a temp table #BridgeMaxLength to hold the max length of all bridges
3. insert some test data
4. Create nonclustered index to boost the performance for large volume of data.

Note: the solution I presented here is inspired from the book: 
	  Inside Microsoft SQL Server 2008: T-SQL Programming
****************************************************************/
Create Table #BridgeP(	ID Int Identity(1,1) Not Null
						, BridgePID int Not Null
						, FromLen Decimal(14,4) Not Null
						, ToLen Decimal(14,4) Not Null
						, Constraint pk_BridgeMeaurement Primary Key (ID)
						, Constraint check_From_Less_than_to Check(FromLen <= ToLen)
					);

Create Table #BridgeMaxLength ( BridgePID int Not Null
								, BridgeMaxLen Decimal(14,4)
							);

Insert into #BridgeP
Values (1, 10.0002, 20.0004)
, (1, 18, 21.0004)
, (1, 11.1234, 20.0002)
, (1, 40, 50.0003)
, (1, 19.11, 35)
, (2, 46.2966, 50.6472)
, (2, 31.7084, 46.1286)
, (2, 178.2604, 190.6864)
, (2, 38.7394, 43.7234)
, (2, 27.266, 43.3031)
, (2, 122.8653, 142.6958)
, (3, 14.7856, 21.6676)
, (3, 67.6054, 68.1884)
, (3, 64.8543, 66.4028)
, (3, 123.331, 134.4094)
, (3, 199.3904, 201.1482)
, (3, 88.1347, 103.9153);

INsert into #BridgeMaxLength
Values (1, 100.0000)
, (2, 200.0000)
, (3, 250.0000);

Create Nonclustered Index idx_Bridge_From_To On  #BridgeP (BridgePID, FromLen, ToLen);
Create Nonclustered Index idx_Bridge_To_From On  #BridgeP (BridgePID, ToLen, FromLen);

/**********************************************************************************
Solution starts here:

1. Since the bridge paint measurement records have overlaps, we first need to idetify
all those paint starting points for a bridge. These starting points must not adjacent
to any end painting points recorded and must be less than any other starting points within
the same packed painting group.
**********************************************************************************/

;With RangeStart
As
(
Select	Distinct A.BridgePID, A.FromLen As RangeSTart
From	#BridgeP A
Where	Not Exists
			(	Select	*
				From	#BridgeP B
				Where	B.BridgePID =  A.BridgePID
						And B.FromLen < A.FromLen
						And B.ToLen >= A.FromLen
			)
)
/**********************************************************************************
2. With the same logic as starting point but in a reversed way, the ending points
can be identified as below:
**********************************************************************************/
, RangeEnd
As
(
Select	Distinct A.BridgePID, A.ToLen As RangeEnd
From	#BridgeP A
Where	Not Exists
		(	Select	*
			From	#BridgeP B
			Where	B.BridgePID =  A.BridgePID
					And B.ToLen > A.ToLen
					And B.FromLen <= A.ToLen
		)
)
/**********************************************************************************
3. Since a bridge should always start from 0 and end witht the maximum length, we add
these 2 points together with all other starting points and end points.
the union all query results must come back with an even number (pairs of start and 
end point) for each bridge.
4. in order to put a starting point and an end point pair into the same group. I used
row_Number function:
(Row_Number() Over(Partition By BridgePID Order by GapStart) - 1) / 2
If the row number is 1, it returns back 0; if the row number is 2, it also return back
0. 
**********************************************************************************/
, GapValue
As
(
Select	BridgePID, GapStart
		, (Row_Number() Over(Partition By BridgePID Order by GapStart) - 1) / 2 As MyGroup
From
		(
			Select	BridgePID, 0.0000 As GapStart
			From	#BridgeMaxLength
				Union All
			Select	BridgePID, RangeStart
			From	RangeStart S
				Union All
			Select	BridgePID, RangeEnd
			From	RangeEnd E
				Union All
			Select	BridgePID, BridgeMaxLen
			From	#BridgeMaxLength
		) X
)

Select		BridgePID, Min(GapStart) As GapStart, Max(GapStart) As GapEnd
From		GapValue
Group By	BridgePID, MyGroup
Having		Min(GapStart) <> Max(GapStart)
Order by	BridgePID, GapStart;


Drop Table #BridgeP;
Drop Table #BridgeMaxLength;