/******************************************************************
Calculate the Period to date values
******************************************************************/
USE [AdventureWorksDW2012]
Go
SELECT	[ProductKey]
		, [OrderDateKey]
		--, Convert(Date,Cast([OrderDateKey] as varchar),112) As OrderDate
		, Sum([SalesAmount]) As SalesAmount
Into	#PeriodToDate
FROM	[AdventureWorksDW2012].[dbo].[FactResellerSales] A
Where	orderdatekey >= 20060101
Group by [ProductKey], [OrderDateKey]
Order by Productkey

Create Unique Clustered Index uidx_Periodtodate on #PeriodTodate (Productkey, orderdatekey);
Create nonClustered Index idx_Periodtodate_2 on #PeriodTodate (orderdatekey) include(SalesAmount);

Select	A.ProductKey, A.OrderDateKey,  B.[CalendarYear], B.[MonthNumberOfYear], A.SalesAmount
		, Sum(A.SalesAmount) Over(	Partition By A.ProductKey, B.[CalendarYear], B.[MonthNumberOfYear]
									Order by A.OrderDateKey
									Rows Between Unbounded Preceding and Current Row
								) As MonthToDateSum
		, Avg(A.SalesAmount) Over(	Partition By A.ProductKey, B.[CalendarYear], B.[MonthNumberOfYear]
									Order by A.OrderDateKey
									Rows Between Unbounded Preceding and Current Row
								) As MonthToDateAVG
		, Sum(A.SalesAmount) Over(	Partition By A.ProductKey, B.[CalendarYear], B.[CalendarQuarter]
									Order by A.OrderDateKey
									Rows Between Unbounded Preceding and Current Row
								) As QuarterToDateSum
		, Avg(A.SalesAmount) Over(	Partition By A.ProductKey, B.[CalendarYear], B.[CalendarQuarter]
									Order by A.OrderDateKey
									Rows Between Unbounded Preceding and Current Row
								) As QuarterToDateAVG
		, Sum(A.SalesAmount) Over(	Partition By A.ProductKey, B.[CalendarYear]
									Order by A.OrderDateKey
									Rows Between Unbounded Preceding and Current Row
								) As YearToDateSum
From	#PeriodToDate A 
			Inner Join
		DimDate B
			on A.OrderDateKey = B.DateKey;

Drop Table #PeriodToDate;