--The performance test code is borrowed and modified from Itzik Ben-Gan's code on performance comparisons.

Use [TechEdNZ2012]
Go

If Object_ID ('dbo.WFBenchmark') is not Null
	
	drop Table dbo.WFBenchmark;

Create Table dbo.WFBenchmark (	ID Int Not Null
								, NumberOfRows Int Not Null
								, RunTimes bigint Not Null
								, Primary Key(ID, NumberOfRows)
							);

If Object_ID('dbo.Sales') Is Not Null
	Drop Table dbo.Sales;

SELECT Top 100000 Row_Number() Over(Order by (Select Null)) As ID
	  ,[ProductKey]
      ,[OrderDateKey]
      ,[ResellerKey]
      ,[EmployeeKey]
      ,[SalesOrderNumber]
      ,[SalesOrderLineNumber]
      ,[OrderQuantity]
      ,[UnitPrice]
      ,[SalesAmount]
Into	dbo.Sales
FROM [AdventureWorksDW2012].[dbo].[FactResellerSalesBig];

Create Unique Clustered index uidx_Sales_1 on dbo.Sales (ID);

Create nonclustered index idx_Sales_2 on dbo.sales (Employeekey) Include (SalesAmount);


Declare		@MaxRows Int = 100000
			, @CurrentNumRows Int = 10000
			, @StepRows Int = 10000
			, @DateTime Datetime2;

While	@CurrentNumRows <= @MaxRows
	Begin
		DBCC FreeProcCache With NO_InfoMSGS;
		DBCC DropCleanBuffers With No_InfoMSGS;

		Set @DateTime = SysDatetime();

		Select	EmployeeKey, SalesAmount
				,	(Select Sum(SalesAmount)
					From	dbo.Sales B
					Where	A.EmployeeKey = B.EmployeeKey
							And B.ID <= A.ID) As Running_Amount
		From	dbo.Sales A
		Where	ID <= @CurrentNumRows
		Order by	EmployeeKey, Running_amount;

		Insert Into dbo.WFBenchmark (ID, NumberOfRows, RunTimes)
		Values (1, @CurrentNumRows,DateDiff(MCS, @DateTime, SYSDATETIME()));

		DBCC FreeProcCache With NO_InfoMSGS;
		DBCC DropCleanBuffers With No_InfoMSGS;

		Set @DateTime = SysDatetime();

		Select	EmployeeKey, SalesAmount
				, Sum(SalesAmount) Over(	Partition By EmployeeKey 
											Order by ID
											Rows Between Unbounded Preceding and Current Row
										) As Running_Amount
		From	dbo.Sales A
		Where	ID <= @CurrentNumRows
		Order by	EmployeeKey, Running_amount;

		Insert Into dbo.WFBenchmark (ID, NumberOfRows, RunTimes)
		Values (2, @CurrentNumRows, DateDiff(MCS, @DateTime, SYSDATETIME()));

		Set	@CurrentNumRows = @CurrentNumRows + @StepRows

	End



	----Trditional Set-Based Running Total Calculation

	--	Select	EmployeeKey, SalesAmount
	--			,	(Select Sum(SalesAmount)
	--				From	dbo.Sales B
	--				Where	A.EmployeeKey = B.EmployeeKey
	--						And B.ID <= A.ID) As Running_Amount
	--	From	dbo.Sales A
	--	--Where	ID < 1000
	--	Order by	EmployeeKey, Running_amount;




	----New SQL server 2012 window function with Order by and Frame Clause
	--	Select	EmployeeKey, SalesAmount
	--			, Sum(SalesAmount) Over(	Partition By EmployeeKey 
	--										Order by ID
	--										Rows Between Unbounded Preceding 
	--												and Current Row
	--									) As Running_Amount
	--	From	dbo.Sales A
	--	Order by	EmployeeKey, Running_amount;